// P_2_2_3_02.pde
// 
// Generative Gestaltung, ISBN: 978-3-87439-759-9

/**
 * form mophing process by connected random agents
 * two forms: circle and line
 * 
 * MOUSE
 * click               : start a new circe
 * position x/y        : direction and speed of floating
 * 
 * KEYS
 * 1-2                 : fill styles
 * 3-4                 : form styles circle/line
 * arrow up/down       : step size +/-
 * f                   : freeze. loop on/off
 * Delete/Backspace    : clear display
 * s                   : save png
 * r                   : start pdf recording
 * e                   : stop pdf recording
 */

//import processing.pdf.*;
//var recordPDF = false;






 
/////////////////////////////////////////////////////////////////
 
autowatch = 1;
//outlets = 1;

var width = 500;
var height = 300;


println("loading P_2_2_3_02");

var formResolution = 15;
var stepSize = 2;
var distortionFactor = 1;
var initRadius = 150;

var centerX, centerY;
var mouseX = 0, mouseY = 0;
var x = new Array(formResolution);
var y = new Array(formResolution);

var filled = false;
var freeze = false;
var mode = 0;

// initalize our processing porting library
var plib = new ProcessingLib(this, draw, width, height);

// call setup 
setup();

function jsevent(event_type, value)
{


  //println("mouseX: " + mouseX + " mouseY: " + mouseY);
}

function setup(){

  // use fullscreen size 
  size(width, height);
  smooth();

  // init form
  centerX = width/2; 
  centerY = height/2;

  var angle = radians(360.0/(formResolution));
  for (var i=0; i<formResolution; i++){
    x[i] = cos(angle*i) * initRadius;
    y[i] = sin(angle*i) * initRadius;  
  }

  stroke(0, 50);
  background(255);
}

function bang()
{
  plib.draw();
}

function draw(){
  
  // floating towards mouse position
  if (mouseX != 0 || mouseY != 0) {
    centerX += (mouseX-centerX) * 0.01;
    centerY += (mouseY-centerY) * 0.01;
  }

  // calculate new points
  for (var i=0; i<formResolution; i++){
    x[i] += random(-stepSize,stepSize);
    y[i] += random(-stepSize,stepSize);
  }

  strokeWeight(0.75);
  if (filled) 
    fill(random(255));
  else 
    noFill();

  if (mode == 0) {
    beginShape();
    // start controlpoint
    curveVertex(x[formResolution-1]+centerX, y[formResolution-1]+centerY);

    // only these points are drawn
    for (var i=0; i<formResolution; i++){
      curveVertex(x[i]+centerX, y[i]+centerY);
    }
    curveVertex(x[0]+centerX, y[0]+centerY);

    // end controlpoint
    curveVertex(x[1]+centerX, y[1]+centerY);
    endShape();
  }

  if (mode == 1) {
    beginShape();
    // start controlpoint
    curveVertex(x[0]+centerX, y[0]+centerY);

    // only these points are drawn
    for (var i=0; i<formResolution; i++){
      curveVertex(x[i]+centerX, y[i]+centerY);
    }

    // end controlpoint
    curveVertex(x[formResolution-1]+centerX, y[formResolution-1]+centerY);
    endShape();
  }

}


function mousePressed() {
  // init forms on mouse position
  centerX = mouseX; 
  centerY = mouseY;

  // circle
  if (mode == 0) {
    centerX = mouseX;
    centerY = mouseY;
    var angle = radians(360/(formResolution));
    var radius = initRadius * random(0.5,1.0);
    for (var i=0; i<formResolution; i++){
      x[i] = cos(angle*i) * radius;
      y[i] = sin(angle*i) * radius;
    }
  } 

  // line
  if (mode == 1) {
    centerX = mouseX;
    centerY = mouseY;
    var radius = initRadius * random(0.5,5.0);
    var angle = random(PI);
    radius = initRadius*4;
    angle = 0;
    
    var x1 = cos(angle) * radius;
    var y1 = sin(angle) * radius;
    var x2 = cos(angle-PI) * radius;
    var y2 = sin(angle-PI) * radius;
    for(var i=0; i<formResolution; i++) {
      x[i] = lerp(x1, x2, i/formResolution);
      y[i] = lerp(y1, y2, i/formResolution);
    }
  }
}

function clear() 
{
  background(255);

}

function keyPressed() {
  if (keyCode == UP) stepSize++;
  if (keyCode == DOWN) stepSize--;
  stepSize = max(stepSize, 1);
  println("stepSize: " + stepSize);
}

function keyReleased() {
  if (key == 's' || key == 'S') saveFrame(timestamp()+"_##.png");
  if (key == DELETE || key == BACKSPACE) background(255);

  if (key == '1') filled = false;
  if (key == '2') filled = true;
  if (key == '3') mode = 0;
  if (key == '4') mode = 1;

  // ------ pdf export ------
  // press 'r' to start pdf recording and 'e' to stop it
  // ONLY by pressing 'e' the pdf is saved to disk!
  if (key =='r' || key =='R') {
    if (recordPDF == false) {
      beginRecord(PDF, timestamp()+".pdf");
      println("recording started");
      recordPDF = true;
      stroke(0, 50);
    }
  } 
  else if (key == 'e' || key =='E') {
    if (recordPDF) {
      println("recording stopped");
      endRecord();
      recordPDF = false;
      background(255); 
    }
  } 

  // switch draw loop on/off
  if (key == 'f' || key == 'F') freeze = !freeze;
  if (freeze == true) noLoop();
  else loop();
}


// timestamp
function timestamp() {
  //Calendar now = Calendar.getInstance();
  return "now";
}


// create our listener object for our window
var mylistener = new JitterListener("MaxSketch",callbackfun);

function callbackfun(event)
{ 
  var x,y,button;

  if (event.eventname=="mouse") {
    // arguments are (x,y,button,cmd,shift,capslock,option,ctrl)
    x = event.args[0];
    y = event.args[1];
    button = event.args[2];
    if (button) 
    {
      mouseX = x;
      mouseY = y;
      mousePressed();
      post("Mouse: " + x + " " + y + "\n");
      mousedown = button;
    }

    //jsketch.position = render.screentoworld(x,y);
  } else if (event.eventname=="mouseidle") {
    x = event.args[0];
    y = event.args[1];
    
    mouseX = x;
    mouseY = y;

  } 
  
  //post("callback: " + event.subjectname + " sent "+ event.eventname + " with (" + event.args + ")\n");
}



/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////



































