/////////////////////////////////////////////////////////////////////////////////////////
// Max processing sketch porting library
// By: Anthony Palomba 

// processinglib classes

// Class Point
function Point(inX, inY) {
 
    this.x = inX;
    this.y = inY;
}



// processinglib varables
var p;
var permutation;
var bStroke = false;
var strokeColor = Array(0.0, 0.0, 0.0, 1.0);
var fillColor = Array(0.0, 0.0, 0.0, 1.0);
var RECTMODE_CORNER = 0;
var RECTMODE_CORNERS = 1
var RECTMODE_CENTER = 2
var RECTMODE_RADIUS = 3
var currentRectMode = RECTMODE_CORNER;
var SHAPEMODE_POINTS = 0
var SHAPEMODE_LINES = 1
var SHAPEMODE_TRIANGLES = 2
var SHAPEMODE_TRIANGLE_FAN = 3
var SHAPEMODE_TRIANGLE_STRIP = 4
var SHAPEMODE_QUADS = 5
var SHAPEMODE_QUAD_STRIP = 6
var SHAPEMODE_NONE = 7
var currentShapeMode = SHAPEMODE_NONE;
var shapePointBuffer = new Array();
var frameCount = 0;
var jitpath = new JitterObject("jit.path");
jitpath.interpmode = "spline";



// processing functions

this.ProcessingLib = function ProcessingLib()
{
  var p = pLibInit();
  return p;
};



function pLibInit()
{

  var p = new Array(512);

  var permutation = [
    151, 160, 137, 91, 90, 15, 131, 13, 201, 95, 96, 53, 194, 233, 7, 225, 140, 36, 103, 30, 69, 142, 8, 99,
    37, 240, 21, 10, 23, 190, 6, 148, 247, 120, 234, 75, 0, 26, 197, 62, 94, 252, 219, 203, 117, 35, 11, 32,
    57, 177, 33, 88, 237, 149, 56, 87, 174, 20, 125, 136, 171, 168, 68, 175, 74, 165, 71, 134, 139, 48, 27, 166,
    77, 146, 158, 231, 83, 111, 229, 122, 60, 211, 133, 230, 220, 105, 92, 41, 55, 46, 245, 40, 244, 102, 143,
    54, 65, 25, 63, 161, 1, 216, 80, 73, 209, 76, 132, 187, 208, 89, 18, 169, 200, 196, 135, 130, 116, 188, 159,
    86, 164, 100, 109, 198, 173, 186, 3, 64, 52, 217, 226, 250, 124, 123, 5, 202, 38, 147, 118, 126, 255, 82,
    85, 212, 207, 206, 59, 227, 47, 16, 58, 17, 182, 189, 28, 42, 223, 183, 170, 213, 119, 248, 152, 2, 44,
    154, 163, 70, 221, 153, 101, 155, 167, 43, 172, 9, 129, 22, 39, 253, 19, 98, 108, 110, 79, 113, 224, 232,
    178, 185, 112, 104, 218, 246, 97, 228, 251, 34, 242, 193, 238, 210, 144, 12, 191, 179, 162, 241, 81, 51,
    145, 235, 249, 14, 239, 107, 49, 192, 214, 31, 181, 199, 106, 157, 184, 84, 204, 176, 115, 121, 50, 45,
    127, 4, 150, 254, 138, 236, 205, 93, 222, 114, 67, 29, 24, 72, 243, 141, 128, 195, 78, 66, 215, 61, 156, 180
  ];

  for (var i = 0; i < 256; i++) {
    p[256 + i] = p[i] = permutation[i];
  }

}

function bang()
{
  drawProcessingScene();
  frameCount++;
}

function drawProcessingScene()
{
  var theImage = null;

  draw();

  mgfx.identity_matrix();
  theImage = new Image(mgfx.pop_group());
  mgfx.image_surface_draw(theImage);

  theImage.tonamedmatrix(outmatrix.name);
  outlet(0, "jit_matrix", outmatrix.name);

  gc();

}

function strokeWeight(x)
{
  mgfx.set_line_width(x);
}

function size(w, h)
{
  pLibInit();

}

function smooth()
{

}

function beginShape(mode)
{
  // points that are created by drawing primitives are added to 
  // shapePointBuffer. Points are then redered in endShape
  shapePointBuffer = new Array();

  currentShapeMode == SHAPEMODE_NONE
  if(arguments.length == 1)
    currentShapeMode = mode;
}

function endShape()
{
  //if(currentShapeMode == SHAPEMODE_NONE)
  jitpath.clear();

  if(shapePointBuffer.length)
  {
        // add our points to our path
    for (var i = 0; i < shapePointBuffer.length; i++) 
    {
      var p1 = shapePointBuffer[i];
      jitpath.append(p1.x, p1.y);
    };

    // calculate our evaluated path
    jitpath.calchandles();
    jitpath.calc_evalmatrix();

    // tell mgraphics to draw our points
    var pathMatrx = new JitterMatrix(jitpath.evalmatrixname);
    for (var j = 0; j < pathMatrx.dim; j++) {

      cell = pathMatrx.getcell(j);
      //post("x:", cell[0], " y:", cell[1], "\n");
      if(j == 0)
        mgfx.move_to(cell[0], cell[1]);
      else
        mgfx.line_to(cell[0], cell[1]);
    }

    mgfx.set_source_rgba(strokeColor[0], strokeColor[1], strokeColor[2], strokeColor[3]);
    mgfx.stroke();
  }
}

function frameRate(rate)
{

}

function rect(x, y, w, h)
{
  var offsetX = 0.0;
  var offsetY = 0.0;

  if(currentRectMode == RECTMODE_CENTER) 
  {
    offsetX = w/2;
    offsetY = h/2;
  }

  mgfx.rectangle(x-offsetX, y-offsetX, w, h);
  strokeFill();
}

function ellipse(x, y, w, h)
{

  mgfx.ellipse(x-w/2, y-h/2, w, h);
  strokeFill();
}

function vertex(x, y)
{
  var p = new Point(x,y);
  shapePointBuffer.push(p);
}

function curveVertex(x, y)
{

  var p = new Point(x,y);
  shapePointBuffer.push(p);
}

function curveVertex2(x1, y1, x2, y2, x3, y3)
{
  mgfx.curve_to(x1, y1, x2, y2, x3, y3);
  strokeFill();
}


function bezierVertex(x2, y2, x3, y3, x4, y4)
{
  mgfx.curve_to(x2, y2, x3, y3, x4, y4);
}

function translate(t_x, t_y)
{
  mgfx.translate(t_x, t_y);
}

function radians(theta)
{
  return theta * Math.PI / 180;
}

function background(rgb) {
  with (mgfx) 
  {
    set_source_rgb(rgb/255, rgb/255, rgb/255);
    rectangle(0, 0, width, height);
    fill();
  }
}

function random(low, high)
{

  //Generates random numbers. Each time the random() function is called, it returns an 
  //unexpected value within the specified range. If only one parameter is passed to the 
  //function, it will return a float between zero and the value of the high parameter. 
  //For example, random(5) returns values between 0 and 5 (starting at zero, and up to, 
  //but not including, 5).

  //If two parameters are specified, the function will return a float with a value between 
  //the two values. For example, random(-5, 10.2) returns values starting at -5 and up to 
  //(but not including) 10.2. To convert a floating-point random number to an integer, use 
  //the int() function. 

  if(arguments.length == 2) 
    return (Math.random()*(high-low)) + low; 
  else 
    return Math.random()*low; 
}

function point(x1, y1)
{
  line(x1, y1, x1+1, y1);
}

function line(x1, y1, x2, y2)
{
  //post("line: " + x1 + " " + y1 + " " + x2 + " " + y2 + "\n");

  with (mgfx) 
  {
    move_to(x1, y1);
    line_to(x2, y2);
    strokeFill();

    //stroke();
  }
}

function println(str) 
{
  post(str + "\n");
}

function paint() {
  TRACE("paint\n"); 
}

function pushMatrix()
{
  mgfx.save();
}

function popMatrix()
{
  mgfx.restore();
}

function pbox(w, h, d)
{
  curMatrix = mgfx.get_matrix();
  x0 = curMatrix[4];
  y0 = curMatrix[5];
  mgfx.rectangle(x0, y0, w, h)  
  mgfx.stroke();
}

function fill(v1, v2, v3, v4)
{
  // if(arguments.length == 4)
  //   mgfx.set_source_rgba(v1/255, v2/255, v3/255, v4/255);
  // else if(arguments.length == 3)
  //   mgfx.set_source_rgba(v1/255, v2/255, v3/255, 1);
  // else if(arguments.length == 2)
  //   mgfx.set_source_rgba(v1/255, v1/255, v1/255, v2/255);
  // else if(arguments.length == 1)
  //   mgfx.set_source_rgba(v1/255, v1/255, v1/255, 1);
  // else 
  //   strokeFill();

  if(arguments.length == 4)
    set_fill_rgba(v1/255, v2/255, v3/255, v4/255);
  else if(arguments.length == 3)
    set_fill_rgba(v1/255, v2/255, v3/255, 1);
  else if(arguments.length == 2)
    set_fill_rgba(v1/255, v1/255, v1/255, v2/255);
  else if(arguments.length == 1)
    set_fill_rgba(v1/255, v1/255, v1/255, 1);
  else 
    strokeFill();

}

function noFill() 
{
  fill(0, 0, 0, 0);
}

function set_stroke_rgba(r, g, b, a)
{
  strokeColor[0] = r;
  strokeColor[1] = g;
  strokeColor[2] = b;
  strokeColor[3] = a;
}
function set_fill_rgba(r, g, b, a)
{
  fillColor[0] = r;
  fillColor[1] = g;
  fillColor[2] = b;
  fillColor[3] = a;
}

function noStroke()
{
  bStroke = false;
}

function stroke(v1, v2, v3, v4) 
{
  bStroke = true;

  // if(arguments.length == 4)
  //   mgfx.set_source_rgba(v1/255, v2/255, v3/255, v4/255);
  // else if(arguments.length == 3)
  //   mgfx.set_source_rgba(v1/255, v2/255, v3/255, 1);
  // else if(arguments.length == 2)
  //   mgfx.set_source_rgba(v1/255, v1/255, v1/255, v2/255);
  // else if(arguments.length == 1)
  //   mgfx.set_source_rgba(v1/255, v1/255, v1/255, 1);

  //mgfx.stroke();

  if(arguments.length == 4)
    set_stroke_rgba(v1/255, v2/255, v3/255, v4/255);
  else if(arguments.length == 3)
    set_stroke_rgba(v1/255, v2/255, v3/255, 1);
  else if(arguments.length == 2)
    set_stroke_rgba(v1/255, v1/255, v1/255, v2/255);
  else if(arguments.length == 1)
    set_stroke_rgba(v1/255, v1/255, v1/255, 1);

}

function strokeFill()
{
  if(bStroke) {
    mgfx.set_source_rgba(strokeColor[0], strokeColor[1], strokeColor[2], strokeColor[3]);
    mgfx.stroke_preserve();
  }
  
  mgfx.set_source_rgba(fillColor[0], fillColor[1], fillColor[2], fillColor[3]);
  mgfx.fill();
}

function noise(x, y, z)
{
  if(arguments.length == 3)
    return noise3d(x, y, z);
  else if(arguments.length == 2)
    return noise2d(x, y);
  else if(arguments.length == 1)
    return noise2d(0, x);
  else 
    return noise2d(0, 0);
}

function fade(t)
{
    return t * t * t * (t * (t * 6 - 15) + 10);
}

function lerp(t, a, b)
{
    return a + t * (b - a);
}

function grad(hash, x, y, z)
{
    // Convert lo 4 bits of hash code into 12 gradient directions.
    var h = hash & 15,
        u = h < 8 ? x : y,
        v = h < 4 ? y : h == 12 || h == 14 ? x : z;
    return ((h & 1) == 0 ? u : -u) + ((h & 2) == 0 ? v : -v);
}

function scale(n)
{
    return (1 + n) / 2;
}

        /** Returns a number between 0 and 1. */
function noise3d(x, y, z)
{
  // Find unit cube that contains point.
  var X = Math.floor(x) & 255,
      Y = Math.floor(y) & 255,
      Z = Math.floor(z) & 255;
  // Find relative x,y,z of point in cube.
  x -= Math.floor(x);
  y -= Math.floor(y);
  z -= Math.floor(z);
  // Compute fade curves for each of x,y,z.
  var u = fade(x),
      v = fade(y),
      w = fade(z);
  // Hash coordinates of the corners.
  var A = p[X    ] + Y, AA = p[A] + Z, AB = p[A + 1] + Z,
      B = p[X + 1] + Y, BA = p[B] + Z, BB = p[B + 1] + Z;

  // Add blended results from 8 corners of cube.
  return scale(
      lerp(
          w,
          lerp(
              v,
              lerp(
                  u,
                  grad(p[AA], x, y, z),
                  grad(p[BA], x - 1, y, z)
              ),
              lerp(
                  u,
                  grad(p[AB], x, y - 1, z),
                  grad(p[BB], x - 1, y - 1, z)
              )
          ),
          lerp(
              v,
              lerp(
                  u,
                  grad(p[AA + 1], x, y, z - 1),
                  grad(p[BA + 1], x - 1, y, z - 1)
              ),
              lerp(
                  u,
                  grad(p[AB + 1], x, y - 1, z - 1),
                  grad(p[BB + 1], x - 1, y - 1, z - 1)
              )
          )
      )
    )
  }

/** Returns a number between 0 and 1. */
function noise2d(x, y)
{
  // Find unit square that contains point.
  var X = Math.floor(x) & 255,
      Y = Math.floor(y) & 255;
  // Find relative x,y of point in square.
  x -= Math.floor(x);
  y -= Math.floor(y);
  // Compute fade curves for each of x,y.
  var u = fade(x),
      v = fade(y);
  // Hash coordinates of the corners.
  var A = p[X    ] + Y, AA = p[A], AB = p[A + 1],
      B = p[X + 1] + Y, BA = p[B], BB = p[B + 1];

  // Add blended results from the corners.
  return scale(
      lerp(
          v,
          lerp(
              u,
              grad(p[AA], x, y, 0),
              grad(p[BA], x - 1, y, 0)
          ),
          lerp(
              u,
              grad(p[AB], x, y - 1, 0),
              grad(p[BB], x - 1, y - 1, 0)
          )
      )
    )
}

function expand(list, newSize)
{
  if(list == undefined)
    return new Array();
  else if((newSize == 0) || (list == null))
    return new Array();
  else
    return list;  // Array already handles expanding
}

function append(list, newItem)
{
  if((list == undefined)||(list == null))  
  {
    var newList = new Array();
    newList.push(newItem);
    return newList;
  }

  list.push(newItem);
  return list;  // Array already handles expanding
}


//The default mode is rectMode(CORNER), which interprets the first two parameters of rect() 
//as the upper-left corner of the shape, while the third and fourth parameters are its width 
//and height.
//rectMode(CORNERS) interprets the first two parameters of rect() as the location of one 
//corner, and the third and fourth parameters as the location of the opposite corner.
//rectMode(CENTER) interprets the first two parameters of rect() as the shape's center point, 
//while the third and fourth parameters are its width and height.
//rectMode(RADIUS) also uses the first two parameters of rect() as the shape's center point, 
//but uses the third and fourth parameters to specify half of the shapes's width and height.
function rectMode(mode)
{

}

function cos(theta)
{
  return Math.cos(theta);
}

function sin(theta)
{
  return Math.sin(theta);
}
